@@ -1072,3 +1072,12 @@ empathy_avatar_chooser_get_image_data (EmpathyAvatarChooser  *chooser,
 	}
 }
 
+void
+empathy_avatar_chooser_set_account (EmpathyAvatarChooser *self,
+				       TpAccount *account)
+{
+	g_return_if_fail (account != NULL);
+
+	avatar_chooser_set_connection (self, tp_account_get_connection (account));
+	g_object_notify (G_OBJECT (self), "connection");
+}
