@@ -231,15 +231,15 @@ empathy_call_handler_class_init (EmpathyCallHandlerClass *klass)
   signals[SRC_PAD_ADDED] =
     g_signal_new ("src-pad-added", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST, 0, NULL, NULL,
-      _empathy_marshal_VOID__OBJECT_UINT,
-      G_TYPE_NONE,
+      _empathy_marshal_BOOLEAN__OBJECT_UINT,
+      G_TYPE_BOOLEAN,
       2, GST_TYPE_PAD, G_TYPE_UINT);
 
   signals[SINK_PAD_ADDED] =
     g_signal_new ("sink-pad-added", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST, 0, NULL, NULL,
-      _empathy_marshal_VOID__OBJECT_UINT,
-      G_TYPE_NONE,
+      _empathy_marshal_BOOLEAN__OBJECT_UINT,
+      G_TYPE_BOOLEAN,
       2, GST_TYPE_PAD, G_TYPE_UINT);
 
   signals[REQUEST_RESOURCE] =
@@ -329,16 +329,32 @@ empathy_call_handler_tf_channel_session_created_cb (TfChannel *tfchannel,
     GST_ELEMENT (conference));
 }
 
+static gboolean
+src_pad_added_error_idle (gpointer data)
+{
+  TfStream *stream = data;
+
+  tf_stream_error (stream, TP_MEDIA_STREAM_ERROR_MEDIA_ERROR,
+      "Could not link sink");
+  g_object_unref (stream);
+
+  return FALSE;
+}
+
 static void
 empathy_call_handler_tf_stream_src_pad_added_cb (TfStream *stream,
   GstPad *pad, FsCodec *codec, EmpathyCallHandler  *handler)
 {
   guint media_type;
+  gboolean retval;
 
   g_object_get (stream, "media-type", &media_type, NULL);
 
   g_signal_emit (G_OBJECT (handler), signals[SRC_PAD_ADDED], 0,
-    pad, media_type);
+      pad, media_type, &retval);
+
+  if (!retval)
+    g_idle_add (src_pad_added_error_idle, g_object_ref (stream));
 }
 
 
@@ -370,6 +386,7 @@ empathy_call_handler_tf_channel_stream_created_cb (TfChannel *tfchannel,
 {
   guint media_type;
   GstPad *spad;
+  gboolean retval;
 
   g_signal_connect (stream, "src-pad-added",
       G_CALLBACK (empathy_call_handler_tf_stream_src_pad_added_cb), handler);
@@ -383,7 +400,11 @@ empathy_call_handler_tf_channel_stream_created_cb (TfChannel *tfchannel,
     "sink-pad", &spad, NULL);
 
   g_signal_emit (G_OBJECT (handler), signals[SINK_PAD_ADDED], 0,
-    spad, media_type);
+      spad, media_type, &retval);
+
+ if (!retval)
+      tf_stream_error (stream, TP_MEDIA_STREAM_ERROR_MEDIA_ERROR,
+          "Could not link source");
 
   gst_object_unref (spad);
 }
@@ -466,7 +487,8 @@ empathy_call_handler_request_cb (EmpathyDispatchOperation *operation,
 }
 
 void
-empathy_call_handler_start_call (EmpathyCallHandler *handler)
+empathy_call_handler_start_call (EmpathyCallHandler *handler,
+    gint64 timestamp)
 {
 
   EmpathyCallHandlerPriv *priv = GET_PRIV (handler);
@@ -515,7 +537,7 @@ empathy_call_handler_start_call (EmpathyCallHandler *handler)
   g_hash_table_insert (request, TP_IFACE_CHANNEL ".TargetHandle", value);
 
   empathy_dispatcher_create_channel (dispatcher, connection,
-    request, empathy_call_handler_request_cb, handler);
+    request, timestamp, empathy_call_handler_request_cb, handler);
 
   g_object_unref (dispatcher);
 }
@@ -533,7 +555,7 @@ empathy_call_handler_stop_call (EmpathyCallHandler *handler)
 
   if (priv->call != NULL)
     {
-      empathy_tp_call_close (priv->call);
+      empathy_tp_call_leave (priv->call);
       g_object_unref (priv->call);
     }
 
