@@ -70,6 +70,7 @@ enum {
   PROP_CONTACT,
   PROP_INCOMING,
   PROP_STATUS,
+  PROP_USER_ACTION_TIME,
 };
 
 /* private structure */
@@ -85,6 +86,7 @@ struct _EmpathyDispatchOperationPriv
   EmpathyContact *contact;
   EmpathyDispatchOperationState status;
   gboolean incoming;
+  gint64 user_action_time;
   gboolean approved;
   gulong invalidated_handler;
   gulong ready_handler;
@@ -132,6 +134,10 @@ empathy_dispatch_operation_set_property (GObject *object,
       case PROP_INCOMING:
         priv->incoming = g_value_get_boolean (value);
         break;
+
+      case PROP_USER_ACTION_TIME:
+        priv->user_action_time = g_value_get_int64 (value);
+        break;
     }
 }
 
@@ -162,6 +168,9 @@ empathy_dispatch_operation_get_property (GObject *object,
       case PROP_STATUS:
         g_value_set_enum (value, priv->status);
         break;
+      case PROP_USER_ACTION_TIME:
+        g_value_set_int64 (value, priv->user_action_time);
+        break;
     }
 }
 
@@ -196,6 +205,9 @@ dispatcher_operation_got_contact_cb (EmpathyTpContactFactory *factory,
   priv->contact = g_object_ref (contact);
   g_object_notify (G_OBJECT (self), "contact");
 
+  /* Ensure to keep the self object alive while the call_when_ready is
+   * running */
+  g_object_ref (self);
   tp_channel_call_when_ready (priv->channel,
     empathy_dispatch_operation_channel_ready_cb, self);
 }
@@ -210,6 +222,13 @@ dispatch_operation_connection_ready (TpConnection *connection,
   EmpathyTpContactFactory *factory;
   TpHandle handle;
 
+  if (error != NULL)
+    goto out;
+
+  if (priv->status >= EMPATHY_DISPATCHER_OPERATION_STATE_CLAIMED)
+    /* no point to get more information */
+    goto out;
+
   handle = tp_channel_get_handle (priv->channel, NULL);
 
   factory = empathy_tp_contact_factory_dup_singleton (priv->connection);
@@ -218,6 +237,8 @@ dispatch_operation_connection_ready (TpConnection *connection,
       dispatcher_operation_got_contact_cb, NULL, NULL, G_OBJECT (self));
 
   g_object_unref (factory);
+out:
+  g_object_unref (self);
 }
 
 static void
@@ -239,11 +260,15 @@ empathy_dispatch_operation_constructed (GObject *object)
 
   if (handle_type == TP_HANDLE_TYPE_CONTACT && priv->contact == NULL)
     {
+      /* Ensure to keep the self object alive while the call_when_ready is
+       * running */
+      g_object_ref (self);
       tp_connection_call_when_ready (priv->connection,
           dispatch_operation_connection_ready, object);
       return;
     }
 
+  g_object_ref (self);
   tp_channel_call_when_ready (priv->channel,
     empathy_dispatch_operation_channel_ready_cb, self);
 }
@@ -341,6 +366,13 @@ empathy_dispatch_operation_class_init (
     EMPATHY_TYPE_DISPATCH_OPERATION_STATE, 0,
     G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_STATUS, param_spec);
+
+  param_spec = g_param_spec_int64 ("user-action-time",
+    "user action time", "The user action time of the operation",
+    G_MININT64, G_MAXINT64, 0,
+    G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, PROP_USER_ACTION_TIME,
+      param_spec);
 }
 
 void
@@ -357,18 +389,16 @@ empathy_dispatch_operation_dispose (GObject *object)
 
   g_object_unref (priv->connection);
 
+  if (priv->ready_handler != 0)
+    g_signal_handler_disconnect (priv->channel_wrapper,
+      priv->ready_handler);
+
   if (priv->channel_wrapper != NULL)
     g_object_unref (priv->channel_wrapper);
 
-  if (priv->ready_handler != 0)
-    g_signal_handler_disconnect (priv->channel_wrapper,
-      priv->invalidated_handler);
-
-
   g_signal_handler_disconnect (priv->channel, priv->invalidated_handler);
   g_object_unref (priv->channel);
 
-
   if (priv->contact != NULL)
     g_object_unref (priv->contact);
 
@@ -433,10 +463,14 @@ empathy_dispatch_operation_channel_ready_cb (TpChannel *channel,
 
   /* The error will be handled in empathy_dispatch_operation_invalidated */
   if (error != NULL)
-    return;
+    goto out;
 
   g_assert (channel == priv->channel);
 
+  if (priv->status >= EMPATHY_DISPATCHER_OPERATION_STATE_CLAIMED)
+    /* no point to get more information */
+    goto out;
+
   /* If the channel wrapper is defined, we assume it's ready */
   if (priv->channel_wrapper != NULL)
     goto ready;
@@ -452,7 +486,7 @@ empathy_dispatch_operation_channel_ready_cb (TpChannel *channel,
         {
           priv->ready_handler = g_signal_connect (chat, "notify::ready",
             G_CALLBACK (empathy_dispatcher_operation_tp_chat_ready_cb), self);
-          return;
+          goto out;
         }
     }
   else if (channel_type == TP_IFACE_QUARK_CHANNEL_TYPE_STREAMED_MEDIA)
@@ -469,32 +503,27 @@ empathy_dispatch_operation_channel_ready_cb (TpChannel *channel,
 ready:
   empathy_dispatch_operation_set_status (self,
     EMPATHY_DISPATCHER_OPERATION_STATE_PENDING);
+out:
+  g_object_unref (self);
 }
 
 EmpathyDispatchOperation *
-empathy_dispatch_operation_new (TpConnection *connection, TpChannel *channel,
-  EmpathyContact *contact, gboolean incoming)
-{
-  return empathy_dispatch_operation_new_with_wrapper (connection, channel,
-    contact, incoming, NULL);
-}
-
-EmpathyDispatchOperation *
-empathy_dispatch_operation_new_with_wrapper (TpConnection *connection,
-  TpChannel *channel, EmpathyContact *contact, gboolean incoming,
-  GObject *wrapper)
+empathy_dispatch_operation_new (TpConnection *connection,
+    TpChannel *channel,
+    EmpathyContact *contact,
+    gboolean incoming,
+    gint64 user_action_time)
 {
   g_return_val_if_fail (connection != NULL, NULL);
   g_return_val_if_fail (channel != NULL, NULL);
 
-  return EMPATHY_DISPATCH_OPERATION (
-    g_object_new (EMPATHY_TYPE_DISPATCH_OPERATION,
+  return g_object_new (EMPATHY_TYPE_DISPATCH_OPERATION,
       "connection", connection,
       "channel", channel,
-      "channel-wrapper", wrapper,
       "contact", contact,
       "incoming", incoming,
-      NULL));
+      "user-action-time", user_action_time,
+      NULL);
 }
 
 void
@@ -672,3 +701,25 @@ empathy_dispatch_operation_is_incoming (EmpathyDispatchOperation *operation)
 
   return priv->incoming;
 }
+
+void
+empathy_dispatch_operation_set_user_action_time (
+    EmpathyDispatchOperation *self,
+    gint64 user_action_time)
+{
+  EmpathyDispatchOperationPriv *priv = GET_PRIV (self);
+
+  priv->user_action_time = user_action_time;
+}
+
+gint64
+empathy_dispatch_operation_get_user_action_time (EmpathyDispatchOperation *self)
+{
+  EmpathyDispatchOperationPriv *priv;
+
+  g_return_val_if_fail (EMPATHY_IS_DISPATCH_OPERATION (self), FALSE);
+
+  priv = GET_PRIV (self);
+
+  return priv->user_action_time;
+}
