@@ -43,19 +43,33 @@
 typedef struct {
 	EmpathyTpContactFactory *factory;
 	TpConnection   *connection;
-	const gchar    *protocol_group;
 
 	TpChannel      *publish;
 	TpChannel      *subscribe;
 	TpChannel      *stored;
-	GHashTable     *members; /* handle -> EmpathyContact */
-	GHashTable     *pendings; /* handle -> EmpathyContact */
-	GHashTable     *groups; /* group name -> TpChannel */
-	GHashTable     *add_to_group; /* group name -> GArray of handles */
+	/* contact handle (TpHandle) => reffed (EmpathyContact *)
+	 *
+	 * Union of:
+	 *  - members of 'subscribe': we receive their presence
+	 *  - RP of 'subscribe': we asked to receive their presence
+	 *  - members of 'publish': we send them our presence
+	 *  - members of 'stored': they are in our roster
+	 */
+	GHashTable     *members;
+	/* contact handle (TpHandle) => reffed (EmpathyContact *)
+	 *
+	 * Contacts which are local-pending in the publish channel but are NOT in
+	 * the members hash table: they asked to receive our presence and we don't
+	 * receive theirs or asked to.
+	 * That's basically the contacts which asked to add us to their contact
+	 * list and we didn't answer it. */
+	GHashTable     *pendings;
+	/* group name: borrowed (const gchar *)  => reffed (TpChannel *) */
+	GHashTable     *groups;
+	/* group name: owned (gchar *) => owned GArray of TpHandle */
+	GHashTable     *add_to_group;
 
 	EmpathyContactListFlags flags;
-
-	TpProxySignalConnection *new_channels_sig;
 } EmpathyTpContactListPriv;
 
 typedef enum {
@@ -263,23 +277,20 @@ tp_contact_list_group_ready_cb (TpChannel *channel,
 static void
 tp_contact_list_group_add_channel (EmpathyTpContactList *list,
 				   const gchar          *object_path,
-				   const gchar          *channel_type,
-				   TpHandleType          handle_type,
-				   guint                 handle)
+				   GHashTable           *properties)
 {
 	EmpathyTpContactListPriv *priv = GET_PRIV (list);
 	TpChannel                *channel;
+	GError *error = NULL;
 
-	/* Only accept server-side contact groups */
-	if (tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_CONTACT_LIST) ||
-	    handle_type != TP_HANDLE_TYPE_GROUP) {
+	channel = tp_channel_new_from_properties (priv->connection,
+				  object_path, properties, &error);
+	if (channel == NULL) {
+		DEBUG ("Failed to create group channel: %s", error->message);
+		g_error_free (error);
 		return;
 	}
 
-	channel = tp_channel_new (priv->connection,
-				  object_path, channel_type,
-				  handle_type, handle, NULL);
-
 	/* Give the ref to the callback */
 	tp_channel_call_when_ready (channel,
 				    tp_contact_list_group_ready_cb,
@@ -361,14 +372,14 @@ tp_contact_list_group_add (EmpathyTpContactList *list,
 }
 
 static void
-tp_contact_list_got_added_members_cb (EmpathyTpContactFactory *factory,
-				      guint                    n_contacts,
-				      EmpathyContact * const * contacts,
-				      guint                    n_failed,
-				      const TpHandle          *failed,
-				      const GError            *error,
-				      gpointer                 user_data,
-				      GObject                 *list)
+got_added_members_cb (EmpathyTpContactFactory *factory,
+		      guint                    n_contacts,
+		      EmpathyContact * const * contacts,
+		      guint                    n_failed,
+		      const TpHandle          *failed,
+		      const GError            *error,
+		      gpointer                 user_data,
+		      GObject                 *list)
 {
 	EmpathyTpContactListPriv *priv = GET_PRIV (list);
 	guint i;
@@ -380,9 +391,8 @@ tp_contact_list_got_added_members_cb (EmpathyTpContactFactory *factory,
 
 	for (i = 0; i < n_contacts; i++) {
 		EmpathyContact *contact = contacts[i];
-		TpHandle handle;
+		TpHandle handle = empathy_contact_get_handle (contact);
 
-		handle = empathy_contact_get_handle (contact);
 		if (g_hash_table_lookup (priv->members, GUINT_TO_POINTER (handle)))
 			continue;
 
@@ -391,17 +401,40 @@ tp_contact_list_got_added_members_cb (EmpathyTpContactFactory *factory,
 				     g_object_ref (contact));
 		g_signal_emit_by_name (list, "members-changed", contact,
 				       0, 0, NULL, TRUE);
-
-		/* This contact is now member, implicitly accept pending. */
-		if (g_hash_table_lookup (priv->pendings, GUINT_TO_POINTER (handle))) {
-			GArray handles = {(gchar *) &handle, 1};
-
-			tp_cli_channel_interface_group_call_add_members (priv->publish,
-				-1, &handles, NULL, NULL, NULL, NULL, NULL);
-		}
 	}
 }
 
+static void
+add_to_members (EmpathyTpContactList *list,
+		GArray *handles)
+{
+	EmpathyTpContactListPriv *priv = GET_PRIV (list);
+	GArray *request;
+	guint i;
+
+	if (handles->len == 0)
+		return;
+
+	request = g_array_new (FALSE, FALSE, sizeof (TpHandle));
+
+	for (i = 0; i < handles->len; i++) {
+		TpHandle handle = g_array_index (handles, TpHandle, i);
+
+		if (g_hash_table_lookup (priv->members, GUINT_TO_POINTER (handle)))
+			continue;
+
+		g_array_append_val (request, handle);
+	}
+
+	if (request->len > 0) {
+			empathy_tp_contact_factory_get_from_handles (priv->factory,
+				request->len, (TpHandle *) request->data,
+				got_added_members_cb, NULL, NULL, G_OBJECT (list));
+	}
+
+	g_array_free (request, TRUE);
+}
+
 static void
 tp_contact_list_got_local_pending_cb (EmpathyTpContactFactory *factory,
 				      guint                    n_contacts,
@@ -474,6 +507,29 @@ tp_contact_list_remove_handle (EmpathyTpContactList *list,
 	}
 }
 
+static void
+remove_from_member_if_needed (EmpathyTpContactList *list,
+			      TpHandle handle)
+{
+	/* remove contact from members if it's not in publish and subscribe */
+	EmpathyTpContactListPriv *priv = GET_PRIV (list);
+	const TpIntSet *members;
+
+	members = tp_channel_group_get_members (priv->subscribe);
+	if (tp_intset_is_member (members, handle))
+		return;
+
+	members = tp_channel_group_get_remote_pending (priv->subscribe);
+	if (tp_intset_is_member (members, handle))
+		return;
+
+	members = tp_channel_group_get_members (priv->publish);
+	if (tp_intset_is_member (members, handle))
+		return;
+
+	tp_contact_list_remove_handle (list, priv->members, handle);
+}
+
 static void
 tp_contact_list_publish_group_members_changed_cb (TpChannel     *channel,
 						  gchar         *message,
@@ -489,6 +545,7 @@ tp_contact_list_publish_group_members_changed_cb (TpChannel     *channel,
 	guint i;
 
 	/* We now send our presence to those contacts, remove them from pendings */
+	add_to_members (list, added);
 	for (i = 0; i < added->len; i++) {
 		tp_contact_list_remove_handle (list, priv->pendings,
 			g_array_index (added, TpHandle, i));
@@ -496,8 +553,10 @@ tp_contact_list_publish_group_members_changed_cb (TpChannel     *channel,
 
 	/* We refuse to send our presence to those contacts, remove from pendings */
 	for (i = 0; i < removed->len; i++) {
-		tp_contact_list_remove_handle (list, priv->pendings,
-			g_array_index (removed, TpHandle, i));
+		TpHandle handle = g_array_index (removed, TpHandle, i);
+
+		tp_contact_list_remove_handle (list, priv->pendings, handle);
+		remove_from_member_if_needed (list, handle);
 	}
 
 	/* Those contacts want our presence, auto accept those that are already
@@ -581,76 +640,59 @@ tp_contact_list_subscribe_group_members_changed_cb (TpChannel     *channel,
 {
 	EmpathyTpContactListPriv *priv = GET_PRIV (list);
 	guint i;
+	GArray *accept;
 
 	/* We now get the presence of those contacts, add them to members */
-	if (added->len > 0) {
-		empathy_tp_contact_factory_get_from_handles (priv->factory,
-			added->len, (TpHandle *) added->data,
-			tp_contact_list_got_added_members_cb, NULL, NULL,
-			G_OBJECT (list));
-	}
+	add_to_members (list, added);
 
 	/* Those contacts refuse to send us their presence, remove from members. */
 	for (i = 0; i < removed->len; i++) {
-		tp_contact_list_remove_handle (list, priv->members,
-			g_array_index (removed, TpHandle, i));
+		remove_from_member_if_needed (list, g_array_index (removed, TpHandle, i));
 	}
 
 	/* We want those contacts in our contact list but we don't get their
 	 * presence yet. Add to members anyway. */
-	if (remote_pending->len > 0) {
-		empathy_tp_contact_factory_get_from_handles (priv->factory,
-			remote_pending->len, (TpHandle *) remote_pending->data,
-			tp_contact_list_got_added_members_cb, NULL, NULL,
-			G_OBJECT (list));
+	add_to_members (list, remote_pending);
+
+	/* Implicitly accept pending request of contacts which are now members. */
+	accept = g_array_new (FALSE, FALSE, sizeof (TpHandle));
+	for (i = 0; i < added->len; i++) {
+		TpHandle handle = g_array_index (added, TpHandle, i);
+
+		if (g_hash_table_lookup (priv->pendings, GUINT_TO_POINTER (handle)) != NULL)
+			g_array_append_val (accept, handle);
 	}
+
+	for (i = 0; i < remote_pending->len; i++) {
+		TpHandle handle = g_array_index (added, TpHandle, i);
+
+		if (g_hash_table_lookup (priv->pendings, GUINT_TO_POINTER (handle)) != NULL)
+			g_array_append_val (accept, handle);
+	}
+
+	tp_cli_channel_interface_group_call_add_members (priv->publish,
+		-1, accept, NULL, NULL, NULL, NULL, NULL);
+
+	g_array_free (accept, TRUE);
 }
 
 static void
-tp_contact_list_new_channel_cb (TpConnection *proxy,
-				const gchar  *object_path,
-				const gchar  *channel_type,
-				guint         handle_type,
-				guint         handle,
-				gboolean      suppress_handler,
-				gpointer      user_data,
-				GObject      *list)
-{
-	tp_contact_list_group_add_channel (EMPATHY_TP_CONTACT_LIST (list),
-					   object_path, channel_type,
-					   handle_type, handle);
-}
-
-static void
-tp_contact_list_list_channels_cb (TpConnection    *connection,
-				  const GPtrArray *channels,
-				  const GError    *error,
-				  gpointer         user_data,
-				  GObject         *list)
+tp_contact_list_store_group_members_changed_cb (TpChannel     *channel,
+						gchar         *message,
+						GArray        *added,
+						GArray        *removed,
+						GArray        *local_pending,
+						GArray        *remote_pending,
+						guint          actor,
+						guint          reason,
+						EmpathyTpContactList *list)
 {
 	guint i;
 
-	if (error) {
-		DEBUG ("Error: %s", error->message);
-		return;
-	}
+	add_to_members (list, added);
 
-	for (i = 0; i < channels->len; i++) {
-		GValueArray  *chan_struct;
-		const gchar  *object_path;
-		const gchar  *channel_type;
-		TpHandleType  handle_type;
-		guint         handle;
-
-		chan_struct = g_ptr_array_index (channels, i);
-		object_path = g_value_get_boxed (g_value_array_get_nth (chan_struct, 0));
-		channel_type = g_value_get_string (g_value_array_get_nth (chan_struct, 1));
-		handle_type = g_value_get_uint (g_value_array_get_nth (chan_struct, 2));
-		handle = g_value_get_uint (g_value_array_get_nth (chan_struct, 3));
-
-		tp_contact_list_group_add_channel (EMPATHY_TP_CONTACT_LIST (list),
-						   object_path, channel_type,
-						   handle_type, handle);
+	for (i = 0; i < removed->len; i++) {
+		remove_from_member_if_needed (list, g_array_index (removed, TpHandle, i));
 	}
 }
 
@@ -699,15 +741,6 @@ tp_contact_list_finalize (GObject *object)
 	G_OBJECT_CLASS (empathy_tp_contact_list_parent_class)->finalize (object);
 }
 
-static gboolean
-received_all_list_channels (EmpathyTpContactList *self)
-{
-	EmpathyTpContactListPriv *priv = GET_PRIV (self);
-
-	return (priv->stored != NULL && priv->publish != NULL &&
-		priv->subscribe != NULL);
-}
-
 static void
 got_list_channel (EmpathyTpContactList *list,
 		  TpChannel *channel)
@@ -724,6 +757,9 @@ got_list_channel (EmpathyTpContactList *list,
 		if (priv->stored != NULL)
 			return;
 		priv->stored = g_object_ref (channel);
+		g_signal_connect (priv->stored, "group-members-changed",
+				  G_CALLBACK (tp_contact_list_store_group_members_changed_cb),
+				  list);
 	} else if (!tp_strdiff (id, "publish")) {
 		if (priv->publish != NULL)
 			return;
@@ -739,12 +775,6 @@ got_list_channel (EmpathyTpContactList *list,
 				  G_CALLBACK (tp_contact_list_subscribe_group_members_changed_cb),
 				  list);
 	}
-
-	if (received_all_list_channels (list) && priv->new_channels_sig != NULL) {
-		/* We don't need to watch NewChannels anymore */
-		tp_proxy_signal_connection_disconnect (priv->new_channels_sig);
-		priv->new_channels_sig = NULL;
-	}
 }
 
 static void
@@ -770,20 +800,16 @@ list_ensure_channel_cb (TpConnection *conn,
 }
 
 static void
-new_channels_cb (TpConnection *conn,
-		 const GPtrArray *channels,
-		 gpointer user_data,
-		 GObject *weak_object)
+iterate_on_channels (EmpathyTpContactList *list,
+		     const GPtrArray *channels)
 {
-	EmpathyTpContactList *list = EMPATHY_TP_CONTACT_LIST (weak_object);
 	guint i;
 
 	for (i = 0; i < channels->len ; i++) {
 		GValueArray *arr = g_ptr_array_index (channels, i);
 		const gchar *path;
 		GHashTable *properties;
-		const gchar *id;
-		TpChannel *channel;
+		TpHandleType handle_type;
 
 		path = g_value_get_boxed (g_value_array_get_nth (arr, 0));
 		properties = g_value_get_boxed (g_value_array_get_nth (arr, 1));
@@ -791,25 +817,51 @@ new_channels_cb (TpConnection *conn,
 		if (tp_strdiff (tp_asv_get_string (properties,
 				TP_IFACE_CHANNEL ".ChannelType"),
 		    TP_IFACE_CHANNEL_TYPE_CONTACT_LIST))
-			return;
+			continue;
 
-		if (tp_asv_get_uint32 (properties,
-				       TP_IFACE_CHANNEL ".TargetHandleType", NULL)
-		    != TP_HANDLE_TYPE_LIST)
-			return;
+		if (tp_asv_get_string (properties, TP_IFACE_CHANNEL ".TargetID") == NULL)
+			continue;
 
-		id = tp_asv_get_string (properties,
-					TP_IFACE_CHANNEL ".TargetID");
-		if (id == NULL)
-			return;
+		handle_type = tp_asv_get_uint32 (properties,
+			TP_IFACE_CHANNEL ".TargetHandleType", NULL);
 
-		channel = tp_channel_new_from_properties (conn, path,
-							  properties, NULL);
-		got_list_channel (list, channel);
-		g_object_unref (channel);
+		if (handle_type != TP_HANDLE_TYPE_GROUP)
+			continue;
+
+		tp_contact_list_group_add_channel (list, path, properties);
 	}
 }
 
+static void
+new_channels_cb (TpConnection *conn,
+		 const GPtrArray *channels,
+		 gpointer user_data,
+		 GObject *weak_object)
+{
+	EmpathyTpContactList *list = EMPATHY_TP_CONTACT_LIST (weak_object);
+
+	iterate_on_channels (list, channels);
+}
+
+static void
+got_channels_cb (TpProxy *conn,
+		 const GValue *out,
+		 const GError *error,
+		 gpointer user_data,
+		 GObject *weak_object)
+{
+	EmpathyTpContactList *list = EMPATHY_TP_CONTACT_LIST (weak_object);
+	const GPtrArray *channels;
+
+	if (error != NULL) {
+		DEBUG ("Get Channels property failed: %s", error->message);
+		return;
+	}
+
+	channels = g_value_get_boxed (out);
+	iterate_on_channels (list, channels);
+}
+
 static void
 conn_ready_cb (TpConnection *connection,
 	       const GError *error,
@@ -824,6 +876,11 @@ conn_ready_cb (TpConnection *connection,
 		goto out;
 	}
 
+	/* Look for existing group channels */
+	tp_cli_dbus_properties_call_get (connection, -1,
+		TP_IFACE_CONNECTION_INTERFACE_REQUESTS, "Channels", got_channels_cb,
+		NULL, NULL, G_OBJECT (list));
+
 	request = tp_asv_new (
 		TP_IFACE_CHANNEL ".ChannelType", G_TYPE_STRING, TP_IFACE_CHANNEL_TYPE_CONTACT_LIST,
 		TP_IFACE_CHANNEL ".TargetHandleType", G_TYPE_UINT, TP_HANDLE_TYPE_LIST,
@@ -832,24 +889,23 @@ conn_ready_cb (TpConnection *connection,
 	/* Watch the NewChannels signal so if ensuring list channels fails (for
 	 * example because the server is slow and the D-Bus call timeouts before CM
 	 * fetches the roster), we have a chance to get them later. */
-	priv->new_channels_sig =
-	  tp_cli_connection_interface_requests_connect_to_new_channels (
+	tp_cli_connection_interface_requests_connect_to_new_channels (
 		priv->connection, new_channels_cb, NULL, NULL, G_OBJECT (list), NULL);
 
 	/* Request the 'stored' list. */
 	tp_asv_set_static_string (request, TP_IFACE_CHANNEL ".TargetID", "stored");
 	tp_cli_connection_interface_requests_call_ensure_channel (priv->connection,
-		-1, request, list_ensure_channel_cb, list, NULL, G_OBJECT (list));
+		G_MAXINT, request, list_ensure_channel_cb, list, NULL, G_OBJECT (list));
 
 	/* Request the 'publish' list. */
 	tp_asv_set_static_string (request, TP_IFACE_CHANNEL ".TargetID", "publish");
 	tp_cli_connection_interface_requests_call_ensure_channel (priv->connection,
-		-1, request, list_ensure_channel_cb, list, NULL, G_OBJECT (list));
+		G_MAXINT, request, list_ensure_channel_cb, list, NULL, G_OBJECT (list));
 
 	/* Request the 'subscribe' list. */
 	tp_asv_set_static_string (request, TP_IFACE_CHANNEL ".TargetID", "subscribe");
 	tp_cli_connection_interface_requests_call_ensure_channel (priv->connection,
-		-1, request, list_ensure_channel_cb, list, NULL, G_OBJECT (list));
+		G_MAXINT, request, list_ensure_channel_cb, list, NULL, G_OBJECT (list));
 
 	g_hash_table_unref (request);
 out:
@@ -860,7 +916,6 @@ static void
 tp_contact_list_constructed (GObject *list)
 {
 	EmpathyTpContactListPriv *priv = GET_PRIV (list);
-	gchar                    *protocol_name = NULL;
 
 	priv->factory = empathy_tp_contact_factory_dup_singleton (priv->connection);
 
@@ -892,25 +947,6 @@ tp_contact_list_constructed (GObject *list)
 
 	tp_connection_call_when_ready (priv->connection, conn_ready_cb,
 		g_object_ref (list));
-
-	tp_cli_connection_call_list_channels (priv->connection, -1,
-					      tp_contact_list_list_channels_cb,
-					      NULL, NULL,
-					      list);
-
-	tp_cli_connection_connect_to_new_channel (priv->connection,
-						  tp_contact_list_new_channel_cb,
-						  NULL, NULL,
-						  list, NULL);
-
-	/* Check for protocols that does not support contact groups. We can
-	 * put all contacts into a special group in that case.
-	 * FIXME: Default group should be an information in the profile */
-	tp_connection_parse_object_path (priv->connection, &protocol_name, NULL);
-	if (!tp_strdiff (protocol_name, "local-xmpp")) {
-		priv->protocol_group = _("People nearby");
-	}
-	g_free (protocol_name);
 }
 
 static void
@@ -1112,10 +1148,6 @@ tp_contact_list_get_all_groups (EmpathyContactList *list)
 		l->data = g_strdup (l->data);
 	}
 
-	if (priv->protocol_group) {
-		ret = g_list_prepend (ret, g_strdup (priv->protocol_group));
-	}
-
 	return ret;
 }
 
@@ -1141,10 +1173,6 @@ tp_contact_list_get_groups (EmpathyContactList *list,
 		}
 	}
 
-	if (priv->protocol_group) {
-		ret = g_list_prepend (ret, g_strdup (priv->protocol_group));
-	}
-
 	return ret;
 }
 
