@@ -52,7 +52,6 @@
 #include "empathy-accounts-dialog.h"
 #include "empathy-import-dialog.h"
 #include "empathy-import-utils.h"
-#include "ephy-spinner.h"
 
 #define DEBUG_FLAG EMPATHY_DEBUG_ACCOUNT
 #include <libempathy/empathy-debug.h>
@@ -264,7 +263,7 @@ accounts_dialog_update_status_infobar (EmpathyAccountsDialog *dialog,
             gtk_info_bar_set_message_type (GTK_INFO_BAR (priv->infobar),
                 GTK_MESSAGE_INFO);
 
-            ephy_spinner_start (EPHY_SPINNER (priv->throbber));
+            gtk_spinner_start (GTK_SPINNER (priv->throbber));
             gtk_widget_show (priv->throbber);
             gtk_widget_hide (priv->image_status);
             break;
@@ -308,7 +307,7 @@ accounts_dialog_update_status_infobar (EmpathyAccountsDialog *dialog,
                message = _("Offline — No Network Connection");
 
             g_object_unref (connectivity);
-            ephy_spinner_stop (EPHY_SPINNER (priv->throbber));
+            gtk_spinner_stop (GTK_SPINNER (priv->throbber));
             gtk_widget_show (priv->image_status);
             gtk_widget_hide (priv->throbber);
             break;
@@ -317,7 +316,7 @@ accounts_dialog_update_status_infobar (EmpathyAccountsDialog *dialog,
             gtk_info_bar_set_message_type (GTK_INFO_BAR (priv->infobar),
                 GTK_MESSAGE_WARNING);
 
-            ephy_spinner_stop (EPHY_SPINNER (priv->throbber));
+            gtk_spinner_stop (GTK_SPINNER (priv->throbber));
             gtk_widget_hide (priv->image_status);
             gtk_widget_hide (priv->throbber);
         }
@@ -328,7 +327,7 @@ accounts_dialog_update_status_infobar (EmpathyAccountsDialog *dialog,
 
       gtk_info_bar_set_message_type (GTK_INFO_BAR (priv->infobar),
           GTK_MESSAGE_WARNING);
-      ephy_spinner_stop (EPHY_SPINNER (priv->throbber));
+      gtk_spinner_stop (GTK_SPINNER (priv->throbber));
       gtk_widget_show (priv->image_status);
       gtk_widget_hide (priv->throbber);
     }
@@ -1664,14 +1663,14 @@ accounts_dialog_add_account (EmpathyAccountsDialog *dialog,
       NULL,
       dialog);
 
-  empathy_signal_connect_weak (account, "notify::display-name",
+  tp_g_signal_connect_object (account, "notify::display-name",
       G_CALLBACK (accounts_dialog_account_display_name_changed_cb),
-      G_OBJECT (dialog));
+      dialog, 0);
 
-  empathy_signal_connect_weak (account, "status-changed",
-      G_CALLBACK (accounts_dialog_connection_changed_cb), G_OBJECT (dialog));
-  empathy_signal_connect_weak (account, "presence-changed",
-      G_CALLBACK (accounts_dialog_presence_changed_cb), G_OBJECT (dialog));
+  tp_g_signal_connect_object (account, "status-changed",
+      G_CALLBACK (accounts_dialog_connection_changed_cb), dialog, 0);
+  tp_g_signal_connect_object (account, "presence-changed",
+      G_CALLBACK (accounts_dialog_presence_changed_cb), dialog, 0);
 
   g_object_unref (settings);
 }
@@ -2047,8 +2046,7 @@ accounts_dialog_build_ui (EmpathyAccountsDialog *dialog)
   gtk_box_pack_start (GTK_BOX (vbox), align, TRUE, TRUE, 0);
 
   /* set up spinner */
-  priv->throbber = ephy_spinner_new ();
-  ephy_spinner_set_size (EPHY_SPINNER (priv->throbber), GTK_ICON_SIZE_SMALL_TOOLBAR);
+  priv->throbber = gtk_spinner_new ();
 
   priv->image_status = gtk_image_new_from_icon_name (
             empathy_icon_name_for_presence (
@@ -2063,10 +2061,10 @@ accounts_dialog_build_ui (EmpathyAccountsDialog *dialog)
   gtk_box_pack_start (GTK_BOX (hbox), priv->label_status, TRUE, TRUE, 0);
 
   /* Tweak the dialog */
-  gtk_window_set_title (GTK_WINDOW (dialog), _("Accounts"));
+  gtk_window_set_title (GTK_WINDOW (dialog), _("Messaging and VoIP Accounts"));
   gtk_window_set_role (GTK_WINDOW (dialog), "accounts");
 
-  gtk_window_set_default_size (GTK_WINDOW (dialog), 640, -1);
+  gtk_window_set_default_size (GTK_WINDOW (dialog), 640, 450);
 
   gtk_window_set_type_hint (GTK_WINDOW (dialog), GDK_WINDOW_TYPE_HINT_DIALOG);
 
@@ -2286,13 +2284,10 @@ empathy_accounts_dialog_show (GtkWindow *parent,
 
 void
 empathy_accounts_dialog_show_application (GdkScreen *screen,
-    GChildWatchFunc application_exit_cb,
-    gpointer user_data,
     TpAccount *selected_account,
     gboolean if_needed,
     gboolean hidden)
 {
-  gint command_pid;
   GError *error = NULL;
   gchar *argv[4] = { NULL, };
   gint i = 0;
@@ -2314,7 +2309,7 @@ empathy_accounts_dialog_show_application (GdkScreen *screen,
 
   argv[i++] = path;
 
-  if (selected_account)
+  if (selected_account != NULL)
     {
       const gchar *account_path;
 
@@ -2336,20 +2331,14 @@ empathy_accounts_dialog_show_application (GdkScreen *screen,
     selected_account == NULL ? "<none selected>" :
       tp_proxy_get_object_path (TP_PROXY (selected_account)));
 
-  gdk_spawn_on_screen (screen, NULL, argv, NULL,
-      G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL,
-      &command_pid, &error);
-  if (error)
+  gdk_spawn_on_screen (screen, NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
+      NULL, NULL, NULL, &error);
+  if (error != NULL)
     {
       g_warning ("Failed to open accounts dialog: %s", error->message);
       g_error_free (error);
     }
 
-  /* XXX: unportable cast to GPid; then again, gdk_spawn_on_screen() seems
-   * unportable since it always takes a gint* for the PID */
-  if (application_exit_cb)
-    g_child_watch_add ((GPid) command_pid, application_exit_cb, NULL);
-
   g_free (account_option);
   g_free (path);
 }
